/*
 * Copyright (c) 2006
 *	Deutsches Zentrum fuer Luft- und Raumfahrt DLR.
 *	All rights reserved.
 *
 * Author: Harti Brandt <hartmut.brandt@dlr.de>
 *
 * Redistribution of this software and documentation and use in source and
 * binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 *
 * 1. Redistributions of source code or documentation must retain the above
 *    copyright notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE AND DOCUMENTATION IS PROVIDED BY FRAUNHOFER FOKUS
 * AND ITS CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * FRAUNHOFER FOKUS OR ITS CONTRIBUTORS  BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: histogram.h 270 2006-03-23 09:41:29Z brandt_h $
 */

#ifndef histogram_h_1142606190
#define histogram_h_1142606190

#include <sys/types.h>
#include <stdint.h>

#define	_HG_DEFINE(CODE, TYPE)						\
struct hg_##CODE##_config {						\
	u_int	maxbins;/* maximum number of bins */			\
	u_int	bins;	/* number of bins */				\
	TYPE	offs;	/* offset */					\
	TYPE	step;	/* step size */					\
};									\
									\
struct hg_##CODE##_data {						\
	struct hg_##CODE##_config config; /* copy of the config */	\
	TYPE	min, max;						\
	uint64_t toobig;						\
	uint64_t toosmall;						\
	uint32_t data[];						\
};									\
									\
/* Reset a histogram */							\
void hg_##CODE##_reset(struct hg_##CODE##_data *);			\
									\
/* New data */								\
void hg_##CODE##_data(struct hg_##CODE##_data *, TYPE);

_HG_DEFINE(dbl, double)
_HG_DEFINE(u32, uint32_t)
_HG_DEFINE(u64, uint64_t)

#undef _HG_DEFINE

/**
 * Declare a histogram structure field
 */
#define	HG_DECLARE(MAXLEN, CODE)					\
	struct {							\
	    struct hg_##CODE##_data hg;					\
	    uint32_t	data[(MAXLEN)];					\
	}

/**
 * Actual size of the histogram
 */
#define	HG_HSIZE(H)	((H)->config.bins * sizeof((H)->data[0]))

/**
 * Generate the case statements for the GET switch for the histogram
 */
#define	_HG_SNMP_GET(NAME, V, H, F)					\
	  case LEAF_##NAME##Len:					\
		(V)->v.uint32 = (H)->config.bins;			\
		return (SNMP_ERR_NOERROR);				\
									\
	  case LEAF_##NAME##Step:					\
		(V)->v.F = (H)->config.step;				\
		return (SNMP_ERR_NOERROR);				\
									\
	  case LEAF_##NAME##Hist:					\
		return (string_get((V), (const u_char *)(H)->data,	\
		    HG_HSIZE((H))));					\
									\
	  case LEAF_##NAME##Offs:					\
		(V)->v.F = (H)->config.offs;				\
		return (SNMP_ERR_NOERROR);				\
									\
	  case LEAF_##NAME##Min:					\
		(V)->v.F = (H)->min;					\
		return (SNMP_ERR_NOERROR);				\
									\
	  case LEAF_##NAME##Max:					\
		(V)->v.F = (H)->max;					\
		return (SNMP_ERR_NOERROR);				\
									\
	  case LEAF_##NAME##TooBig:					\
		(V)->v.counter64 = (H)->toobig;				\
		return (SNMP_ERR_NOERROR);				\
									\
	  case LEAF_##NAME##TooSmall:					\
		(V)->v.counter64 = (H)->toosmall;			\
		return (SNMP_ERR_NOERROR);

/**
 * Generate the cases for the GET switch of the configuration
 */
#define	_HG_SNMP_GETC(NAME, V, H, F)					\
	  case LEAF_##NAME##LenC:					\
		(V)->v.F = (H)->bins;					\
		return (SNMP_ERR_NOERROR);				\
									\
	  case LEAF_##NAME##StepC:					\
		(V)->v.F = (H)->step;					\
		return (SNMP_ERR_NOERROR);				\
									\
	  case LEAF_##NAME##OffsC:					\
		(V)->v.F = (H)->offs;					\
		return (SNMP_ERR_NOERROR);

/**
 * Generate the cases for the SET switch of the configuration
 */
#define	_HG_SNMP_SETC(NAME, V, H, C, F)					\
	  case LEAF_##NAME##LenC:					\
		if ((V)->v.uint32 == 0 || (V)->v.uint32 > (H)->maxbins)	\
			return (SNMP_ERR_WRONG_VALUE);			\
		memcpy(&(C)->scratch->int1, &(H)->bins,			\
		    sizeof((H)->bins));					\
		(H)->bins = (V)->v.uint32;				\
		return (SNMP_ERR_NOERROR);				\
									\
	  case LEAF_##NAME##StepC:					\
		if ((V)->v.F <= 0)					\
			return (SNMP_ERR_WRONG_VALUE);			\
		memcpy(&(C)->scratch->int1, &(H)->step,			\
		    sizeof((H)->step));					\
		(H)->step = (V)->v.F;					\
		return (SNMP_ERR_NOERROR);				\
									\
	  case LEAF_##NAME##OffsC:					\
		memcpy(&(C)->scratch->int1, &(H)->offs,			\
		    sizeof((H)->offs));					\
		(H)->offs = (V)->v.F;					\
		return (SNMP_ERR_NOERROR);

/**
 * Generate the cases for the COMMIT switch
 */
#define	_HG_SNMP_COMMITC(NAME)						\
	  case LEAF_##NAME##LenC:					\
	  case LEAF_##NAME##StepC:					\
	  case LEAF_##NAME##OffsC:					\
		return (SNMP_ERR_NOERROR);

/**
 * Generate the cases for the ROLLBACK switch
 */
#define	_HG_SNMP_ROLLBACKC(NAME, H, C)					\
	  case LEAF_##NAME##LenC:					\
		memcpy(&(H)->bins, &(C)->scratch->int1,			\
		    sizeof((H)->bins));					\
		return (SNMP_ERR_NOERROR);				\
									\
	  case LEAF_##NAME##StepC:					\
		memcpy(&(H)->step, &(C)->scratch->int1,			\
		    sizeof((H)->step));					\
		return (SNMP_ERR_NOERROR);				\
									\
	  case LEAF_##NAME##OffsC:					\
		memcpy(&(H)->offs, &(C)->scratch->int1,			\
		    sizeof((H)->offs));					\
		return (SNMP_ERR_NOERROR);

/**
 * Now create all the macros
 */
#define	HG_U32_SNMP_GET(NAME, V, H)	_HG_SNMP_GET(NAME, V, H, uint32)
#define	HG_U64_SNMP_GET(NAME, V, H)	_HG_SNMP_GET(NAME, V, H, counter64)

#define HG_U32_SNMP_GETC(NAME, V, H)	_HG_SNMP_GETC(NAME, V, H, uint32)
#define HG_U64_SNMP_GETC(NAME, V, H)	_HG_SNMP_GETC(NAME, V, H, counter64)

#define	HG_U32_SNMP_SETC(NAME, V, H, C) _HG_SNMP_SETC(NAME, V, H, C, uint32)
#define	HG_U64_SNMP_SETC(NAME, V, H, C) _HG_SNMP_SETC(NAME, V, H, C, counter64)

#define	HG_U32_SNMP_COMMITC(NAME)	_HG_SNMP_COMMITC(NAME)
#define	HG_U64_SNMP_COMMITC(NAME)	_HG_SNMP_COMMITC(NAME)

#define	HG_U32_SNMP_ROLLBACKC(NAME, H, C)	_HG_SNMP_ROLLBACKC(NAME, H, C)
#define	HG_U64_SNMP_ROLLBACKC(NAME, H, C)	_HG_SNMP_ROLLBACKC(NAME, H, C)

#endif
