/*
 * Copyright (c) 2005
 *	Deutsches Zentrum fuer Luft- und Raumfahrt DLR.
 *	All rights reserved.
 *
 * Author: Harti Brandt <hartmut.brandt@dlr.de>
 *
 * Redistribution of this software and documentation and use in source and
 * binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 *
 * 1. Redistributions of source code or documentation must retain the above
 *    copyright notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE AND DOCUMENTATION IS PROVIDED BY FRAUNHOFER FOKUS
 * AND ITS CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * FRAUNHOFER FOKUS OR ITS CONTRIBUTORS  BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: rng.h 222 2006-02-01 16:37:08Z brandt_h $
 */

#ifndef rng_h_1115135178
#define rng_h_1115135178

struct mtrand;

struct rng_exp;
struct rng_normal;
struct rng_pareto;
struct rng_uni;

struct rng_exp *rng_exp_init(struct mtrand *, double);
struct rng_exp *rng_exp_modify(struct rng_exp *, struct mtrand *, double);
void rng_exp_free(struct rng_exp *);
struct mtrand *rng_exp_mt(struct rng_exp *);
double rng_exp(struct rng_exp *);
double rng_exp_mean(const struct rng_exp *);

struct rng_norm *rng_norm_init(struct mtrand *, double, double);
struct rng_norm *rng_norm_modify(struct rng_norm *, struct mtrand *,
    double, double);
void rng_norm_free(struct rng_norm *);
struct mtrand *rng_norm_mt(struct rng_norm *);
double rng_norm(struct rng_norm *);
double rng_norm_mean(const struct rng_norm *);
double rng_norm_dev(const struct rng_norm *);

struct rng_pareto *rng_pareto_init(struct mtrand *, double, double, double);
struct rng_pareto *rng_pareto_modify(struct rng_pareto *, struct mtrand *,
    double, double, double);
void rng_pareto_free(struct rng_pareto *);
struct mtrand *rng_pareto_mt(struct rng_pareto *);
double rng_pareto(struct rng_pareto *);
double rng_pareto_shape(const struct rng_pareto *);
double rng_pareto_scale(const struct rng_pareto *);
double rng_pareto_shift(const struct rng_pareto *);

struct rng_uni *rng_uni_init(struct mtrand *, double, double);
struct rng_uni *rng_uni_modify(struct rng_uni *, struct mtrand *, double,
    double);
void rng_uni_free(struct rng_uni *);
struct mtrand *rng_uni_mt(struct rng_uni *);
double rng_uni(struct rng_uni *);
double rng_uni_min(const struct rng_uni *);
double rng_uni_max(const struct rng_uni *);

#endif
