/*
 * Copyright (c) 2005-2006
 *	Deutsches Zentrum fuer Luft- und Raumfahrt DLR.
 *	All rights reserved.
 *
 * Author: Harti Brandt <hartmut.brandt@dlr.de>
 *
 * Redistribution of this software and documentation and use in source and
 * binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 *
 * 1. Redistributions of source code or documentation must retain the above
 *    copyright notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE AND DOCUMENTATION IS PROVIDED BY FRAUNHOFER FOKUS
 * AND ITS CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * FRAUNHOFER FOKUS OR ITS CONTRIBUTORS  BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: shmem.h 537 2006-09-08 09:18:57Z brandt_h $
 */

#ifndef shmem_h_1117708985
#define shmem_h_1117708985

#include <sys/types.h>
#include <stdint.h>

/* file descriptor to pass the shared memory from the parent to the child */
#define	SHMEM_FD	3

/*
 * Common header for the shared memory between the SNMP module and
 * a simulation process.
 */
struct acm_shmem {
	uint8_t		running;	/* running flag */
	uint32_t	debug_level;	/* debug flags */
	size_t		memsize;	/* size of the total block */
};

/*
 * Common header for a single instance
 */
struct acm_shmem_instance {
	uint8_t		used;		/* used entry */
	uint8_t		town;		/* owned by traffic generator */
	uint32_t	error;		/* error flag */
	void		*tdata;		/* traffic generator data */
	void		*cdata;		/* controller data */
};

/**
 * command structure
 */
enum acm_command_code {
	ACM_START_STOP,		/* start/stop instances */
	ACM_IPLIB_DEBUG,	/* change debug settings */
	ACM_RESET_INST,		/* reset instance */
	ACM_RESET_GLOB,		/* reset globally */
	ACM_RESET_ALL,		/* reset all instances and global */
	ACM_INIT		/* initialize traffic module */
};
struct acm_command {
	u_int		len;		/* length of this command */
	enum acm_command_code cmd;	/* the command */
	union {
	    struct {
	    }		start_stop;
	    uint32_t	iplib_debug;
	    struct {
		u_int	inst;		/* instance number */
		u_int	proto;		/* which protocol */
		u_int	pindex;		/* protocol index */
	    }		reset_inst;
	}		u;
};
/* command handler type */
typedef void (*acm_command_f)(const struct acm_command *);

void *open_shmem(acm_command_f);

#endif
